"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var ReadModel = /** @class */ (function () {
    function ReadModel(framework) {
        var _this = this;
        this.framework = framework;
        /**
           * @type {Function} readModelGenerator - transform an event stream into a json object
           * @param {Object} readModel - a json object representing the state of a given model
           * @param {Function} reducer - a function which reduces events into a read model state object
           * @param {Object[]} events - events from an eventStore contract
           */
        this.readModelGenerator = function (readModel, reducer, events) {
            if (!readModel) {
                throw Error('readModelGenerator was passed a broken readModel!' + JSON.stringify(readModel));
            }
            if (!reducer) {
                throw Error('readModelGenerator was passed a broken reducer!' + JSON.stringify(reducer));
            }
            if (!events) {
                throw Error('readModelGenerator was passed a broken events!' + JSON.stringify(events));
            }
            events.forEach(function (event) {
                readModel = reducer(readModel, event);
            });
            if (readModel === undefined) {
                throw Error('the reducer function returned undefined instead of a valid state object.');
            }
            return readModel;
        };
        /**
      * @type {Function} maybeSyncReadModel - maybe update a json read model if it has new events
      * @param {Contract} eventStore - a contract instance which is an Event Store
      * @param {Object} readModel - a json object representing the state of a given model
      * @param {Function} reducer - a function which reduces events into a read model state object
      * @return {Promise<ReadModel, Error>} json object representing the state of a ReadModel for an EventStore
      */
        this.maybeSyncReadModel = function (eventStore, fromAddress, readModel, reducer) { return __awaiter(_this, void 0, void 0, function () {
            var _this = this;
            var eventCount;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!readModel) {
                            throw Error('Cannot sync because readModel is broken: ' + JSON.stringify(readModel));
                        }
                        return [4 /*yield*/, eventStore.eventCount.call({
                                from: fromAddress,
                            })];
                    case 1:
                        eventCount = (_a.sent()).toNumber();
                        return [2 /*return*/, this.framework.Persistence.LocalStore
                                .getItem(readModel.readModelStoreKey)
                                .then(function (_readModel) { return __awaiter(_this, void 0, void 0, function () {
                                var startIndex, events;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            if (!_readModel) {
                                                // console.log('This read model has not been stored yet, lets initialize its index values')
                                                _readModel = readModel;
                                                _readModel.readModelStoreKey = readModel.readModelType + ":" + readModel.contractAddress;
                                            }
                                            if (!(_readModel.lastEvent === eventCount)) return [3 /*break*/, 1];
                                            _readModel = this.framework.Persistence.LocalStore.setItem(_readModel.readModelStoreKey, _readModel);
                                            return [3 /*break*/, 3];
                                        case 1:
                                            startIndex = _readModel.lastEvent !== null ? _readModel.lastEvent + 1 : 0;
                                            return [4 /*yield*/, this.framework.EventStore.readFSAs(eventStore, fromAddress, startIndex)];
                                        case 2:
                                            events = _a.sent();
                                            _readModel = this.readModelGenerator(_readModel, reducer, events);
                                            if (!_readModel) {
                                                throw Error('failed to get a read model from the generator!');
                                            }
                                            _a.label = 3;
                                        case 3: return [2 /*return*/, this.framework.Persistence.LocalStore.setItem(_readModel.readModelStoreKey, _readModel)];
                                    }
                                });
                            }); })];
                }
            });
        }); };
        this.getCachedReadModel = function (eventStore, fromAddress, readModel, reducer) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        readModel.contractAddress = eventStore.address;
                        readModel.readModelStoreKey = readModel.readModelType + ":" + readModel.contractAddress;
                        return [4 /*yield*/, this.maybeSyncReadModel(eventStore, fromAddress, readModel, reducer)];
                    case 1:
                        readModel = _a.sent();
                        return [2 /*return*/, readModel];
                }
            });
        }); };
    }
    return ReadModel;
}());
exports.ReadModel = ReadModel;
//# sourceMappingURL=ReadModel.js.map